//****************************************************
//***** m6955/ENCODER/LCD TEST *********************
//*************************** By Minarai Shokunin ****
//*************************** 2014/02/02          ****
//****************************************************

#include "p24FJ64GA002.h"
#include "spilcd.h"
#include "m6955.h"

// ************ CONFIGRATION BITS ********************
_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & COE_OFF & ICS_PGx1 & FWDTEN_OFF )
_CONFIG2(IESO_OFF & FNOSC_FRCPLL & FCKSM_CSDCMD & OSCIOFNC_ON & IOL1WAY_OFF & I2C1SEL_PRI & POSCMOD_NONE)


extern unsigned char VirtualScreen[6][128];

// ************ prototypes ***************************
extern void SPILCD_init();
extern void WriteData(int,int);
extern void Copy2SPILCD(void);
extern void GR_SCALE(int);
extern void GR_FILL(unsigned char);
extern void GR_PSET(int,int,unsigned char);
extern void GR_LINE(int,int,int,int,unsigned char);
extern void GR_PUTCH(int,int,unsigned char, unsigned char);
extern void GR_BOXFILL(int,int,int,int,unsigned char);
extern void GR_BOX(int,int,int,int,unsigned char);
extern void GR_PUTNUM(int,int,int, unsigned char);
extern void GR_PUTSTR(int,int,unsigned char *, unsigned char);
extern void I2c_Reg_Write(unsigned int,unsigned char *,unsigned int );
extern void I2c_Reg_Read(unsigned int,unsigned char *,unsigned int);

extern void m6955_init(void);
extern void m6955_setband(int);
extern void m6955_setfreq(long);


void wait_some(int);
void Put_Freq(int,int,long, unsigned char);

void __attribute__((interrupt, auto_psv)) _T1Interrupt(void);

#define LED_R 	LATBbits.LATB6
#define LED_G 	LATBbits.LATB7
#define LED_B 	LATBbits.LATB8
#define SW 		PORTBbits.RB9
#define ENC_A 	PORTBbits.RB10
#define ENC_B 	PORTBbits.RB5

#define CW		1
#define	CCW		-1
#define HOLD	0
#define ON		1
#define OFF		0
#define TH_SLOW	2500


//************************ LW     MW    SW1   SW2   SW3   SW4   SW5   SW6   SW7    SW8   SW9    SW10   SW12   SW13     FM2
const long BAND_FREQL[] = {150 ,  522, 4700, 3200, 4700, 5700, 6800, 9200, 11400, 13500, 15000, 17400, 21400, 11400, 76000};
const long BAND_FREQM[] = {150 ,  810, 4700, 3200, 4700, 5700, 6800, 9200, 11400, 13500, 15000, 17400, 21400, 11400, 81300};
const long BAND_FREQH[] = {285 , 1620, 5600, 4100, 5600, 6400, 7600,10000, 12200, 14300, 15900, 17900, 21900, 17900, 108000};
const long BAND_STEP[] =  {  3 ,    9,    5,    5,    5,    5,    5,    5,     5,     5,     5,     5,     5,     5,   100};
const char BAND_NAME[][4]= {"LW","MW","SW1","SW2","SW3","SW4","SW5","SW6","SW7","SW8","SW9","SW10","SW11","SW12","SW13","FM"};



int nEncoder_state;
int nEncoder_dir;
int nEncoder_count;
int nEncoder_sw;
int nEncoder_data_valid;
int nEncoder_pushed = 0;

int nGlBand = AMMODE;		// AM mode
long lGlFreq	= 810;			// 810KHz
int nGlVolume;

// *********************************************
// ************ MAIN ***************************
// *********************************************

int main(void)
{
	int i;
	int freq_scale = 1;
	// ************ CPU Initialize **********
	
	_RCDIV = 0;						// PostScaler 32MHz
	AD1PCFG = 0xffff;				// All digital
	TRISB = 0b0000011000110000;			// PORTB 4,5,9 are input / others outout
	PORTB = 0xffff;
	
	nEncoder_state = 0;
	nEncoder_count;
	nEncoder_dir = HOLD;
	nEncoder_data_valid = 0;

	SPILCD_init();
	GR_FILL(0x00);

	GR_SCALE(1);
	GR_PUTSTR(10,0,"AITENDO M6955",1);
	GR_SCALE(2);
	GR_PUTSTR(10,10,"AMMODE",1);
	Copy2SPILCD();


	// *********** M6955 initialize *******************
	m6955_init();

	// *********** Interval timer Itinialize ***************
    T1CONbits.TCKPS 	= 1;   // Timer1 Pre scaler 1:8
	PR1 				= 200; // 100us interval timer
    T1CONbits.TON   	= 1;    // Timer1 ON
 	IEC0bits.T1IE = 1;  // Timer1 interrupt enable

	// *********** Main loop start here **************
	m6955_setband(nGlBand);
	m6955_setfreq(lGlFreq);	
	GR_PUTSTR(10,32,"      ",0);
	Put_Freq(10,32,lGlFreq,1);
	Copy2SPILCD();

	while (1) {
		// ********** GR[_ *************
		if (nEncoder_pushed == 1) {
			GR_FILL(0x00);
			GR_SCALE(1);
			GR_PUTSTR(10,0,"AITENDO M6955",1);
			GR_SCALE(2);
			if (nGlBand == AMMODE) {
				nGlBand = FMMODE;
				GR_PUTSTR(10,10,"FMMODE",1);
			}else{
				nGlBand = AMMODE;
				GR_PUTSTR(10,10,"AMMODE",1);
			}
			lGlFreq = BAND_FREQM[nGlBand];
			m6955_setband(nGlBand);
			m6955_setfreq(lGlFreq);	
			GR_BOXFILL(10,30,110,20,0);
			Put_Freq(10,30,lGlFreq,1);
			Copy2SPILCD();
			nEncoder_pushed = 0;
		}

		// ********** GR[_] *************
		if (nEncoder_data_valid == 1) {
			freq_scale = 1;
			if (nEncoder_count < TH_SLOW) {
				freq_scale = 3;
			}
			if (nEncoder_dir == CW) {
				lGlFreq += BAND_STEP[nGlBand]*freq_scale;
				if (lGlFreq > BAND_FREQH[nGlBand]) {
					lGlFreq -= BAND_STEP[nGlBand]*freq_scale;
				}
			}else{
				lGlFreq -= BAND_STEP[nGlBand]*freq_scale;
				if (lGlFreq < BAND_FREQL[nGlBand]) {		
					lGlFreq += BAND_STEP[nGlBand]*freq_scale;
				}
			}
			m6955_setband(nGlBand);
			m6955_setfreq(lGlFreq);
			nEncoder_data_valid = 0;
			Put_Freq(10,30,lGlFreq,1);
			Copy2SPILCD();
		}
	}
}

//********** Simple wait loop *************
void wait_some(int t)
{
	int i,j;
	for (i=0;i